import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.Arrays;
import java.util.List;
import javax.xml.crypto.dsig.CanonicalizationMethod;
import javax.xml.crypto.dsig.DigestMethod;
import javax.xml.crypto.dsig.Reference;
import javax.xml.crypto.dsig.SignatureMethod;
import javax.xml.crypto.dsig.SignedInfo;
import javax.xml.crypto.dsig.Transform;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMSignContext;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.crypto.dsig.keyinfo.KeyInfoFactory;
import javax.xml.crypto.dsig.keyinfo.KeyValue;
import javax.xml.crypto.dsig.spec.C14NMethodParameterSpec;
import javax.xml.crypto.dsig.spec.DigestMethodParameterSpec;
import javax.xml.crypto.dsig.spec.SignatureMethodParameterSpec;
import javax.xml.crypto.dsig.spec.TransformParameterSpec;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

public class XMLSigner {
    public static void main(String[] args) throws Exception {
        // STEP 1: Create an XMLSignatureFactory
        XMLSignatureFactory xmlSignatureFactory = 
                XMLSignatureFactory.getInstance();

        // STEP 2: Create key for signing
        // Create DSA KeyPair
        KeyPairGenerator keyPairGenerator = KeyPairGenerator.
                getInstance("DSA");
        keyPairGenerator.initialize(512);
        KeyPair keyPair = keyPairGenerator.generateKeyPair();

        // Create private key
        PrivateKey privateKey = keyPair.getPrivate();
        
        // Create KeyValue containing our DSA PublicKey 
        PublicKey publicKey = keyPair.getPublic();
        KeyInfoFactory keyInfoFactory = xmlSignatureFactory.
                getKeyInfoFactory();
        KeyValue keyValue = keyInfoFactory.
                newKeyValue(publicKey);
        
        // Create buat KeyInfo and add KeyValue 
        List keyValues = Arrays.asList(keyValue);
        KeyInfo keyInfo = keyInfoFactory.
                newKeyInfo(keyValues);
        

        // STEP 3: Read the XML document to be signed 
        // and prepare the Node
        String inputFilename = "TransactionDocument.xml";
        InputStream inputStream = new 
                FileInputStream(inputFilename);
        DocumentBuilderFactory documentBuilderFactory = 
                DocumentBuilderFactory
                .newInstance();
        DocumentBuilder builder = documentBuilderFactory.
                newDocumentBuilder();
        Document document = builder.parse(inputStream);
        Node node = document.getDocumentElement();

        
        // STEP 4: Prepare the XMLSignatureFactory
        // Create DigestMethod 
        String digestAlgorithm = DigestMethod.SHA1;
        DigestMethodParameterSpec digestParams = null;
        DigestMethod digestMethod = xmlSignatureFactory.
                newDigestMethod(
                digestAlgorithm, digestParams); 

        // Create Transform
        String transformAlgorithm = Transform.ENVELOPED;
        TransformParameterSpec transformParams = null; 
        Transform transform = xmlSignatureFactory.
                newTransform(
                transformAlgorithm, transformParams);

        // Create Reference
        String uri = "";
        List transforms = Arrays.asList(transform);
        String type = null;
        String id = null;
        Reference reference = xmlSignatureFactory.
                newReference(uri, digestMethod, 
                transforms, type, id);

        // Create CanonicalizationMethod
        String canonAlgorithm = CanonicalizationMethod
                .INCLUSIVE_WITH_COMMENTS;
        C14NMethodParameterSpec canonParams = null;
        CanonicalizationMethod canonicalizationMethod = 
                xmlSignatureFactory.
                newCanonicalizationMethod(
                canonAlgorithm, canonParams);
        
        // Create SignatureMethod
        String signatureAlgorithm = SignatureMethod.DSA_SHA1;
        SignatureMethodParameterSpec signatureParams = null;
        SignatureMethod signatureMethod = 
                xmlSignatureFactory.newSignatureMethod(
                signatureAlgorithm, signatureParams);
        
        // Create SignedInfo
        List references = Arrays.asList(reference);
        SignedInfo signedInfo = xmlSignatureFactory.
                newSignedInfo(canonicalizationMethod, 
                signatureMethod, references);
        
        
        // STEP 5: Create DOMSignContext and XMLSignature
        DOMSignContext signContext = new 
                DOMSignContext(privateKey, node);
        // Create XML signature
        XMLSignature signature = xmlSignatureFactory.
                newXMLSignature(signedInfo, keyInfo);
        

        // STEP 6: Sign the DOMSignContext
        signature.sign(signContext);
        
        
        // STEP 7: Write the signed XML to file
        String outputFilename = "SignedDocument.xml";
        OutputStream outputStream = new 
                FileOutputStream(outputFilename);
        Source xmlSource = new DOMSource(document);
        Result outputTarget = new StreamResult(outputStream);
        TransformerFactory transformerFactory = 
                TransformerFactory.newInstance();
        Transformer transformer = 
                transformerFactory.newTransformer();
        transformer.transform(xmlSource, outputTarget);
    }
}
